package javax.swing.text;

import java.awt.ComponentOrientation;
import java.awt.Point;
import java.awt.Rectangle;
import javax.swing.UIManager;
import sun.awt.SunToolkit;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.io.Writer;
import java.io.Reader;
import java.io.OutputStreamWriter;
import java.io.OutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.InputStream;
import javax.swing.Action;

public class DefaultEditorKit extends EditorKit
{
    public static final String EndOfLineStringProperty = "__EndOfLine__";
    public static final String insertContentAction = "insert-content";
    public static final String insertBreakAction = "insert-break";
    public static final String insertTabAction = "insert-tab";
    public static final String deletePrevCharAction = "delete-previous";
    public static final String deleteNextCharAction = "delete-next";
    public static final String deleteNextWordAction = "delete-next-word";
    public static final String deletePrevWordAction = "delete-previous-word";
    public static final String readOnlyAction = "set-read-only";
    public static final String writableAction = "set-writable";
    public static final String cutAction = "cut-to-clipboard";
    public static final String copyAction = "copy-to-clipboard";
    public static final String pasteAction = "paste-from-clipboard";
    public static final String beepAction = "beep";
    public static final String pageUpAction = "page-up";
    public static final String pageDownAction = "page-down";
    static final String selectionPageUpAction = "selection-page-up";
    static final String selectionPageDownAction = "selection-page-down";
    static final String selectionPageLeftAction = "selection-page-left";
    static final String selectionPageRightAction = "selection-page-right";
    public static final String forwardAction = "caret-forward";
    public static final String backwardAction = "caret-backward";
    public static final String selectionForwardAction = "selection-forward";
    public static final String selectionBackwardAction = "selection-backward";
    public static final String upAction = "caret-up";
    public static final String downAction = "caret-down";
    public static final String selectionUpAction = "selection-up";
    public static final String selectionDownAction = "selection-down";
    public static final String beginWordAction = "caret-begin-word";
    public static final String endWordAction = "caret-end-word";
    public static final String selectionBeginWordAction = "selection-begin-word";
    public static final String selectionEndWordAction = "selection-end-word";
    public static final String previousWordAction = "caret-previous-word";
    public static final String nextWordAction = "caret-next-word";
    public static final String selectionPreviousWordAction = "selection-previous-word";
    public static final String selectionNextWordAction = "selection-next-word";
    public static final String beginLineAction = "caret-begin-line";
    public static final String endLineAction = "caret-end-line";
    public static final String selectionBeginLineAction = "selection-begin-line";
    public static final String selectionEndLineAction = "selection-end-line";
    public static final String beginParagraphAction = "caret-begin-paragraph";
    public static final String endParagraphAction = "caret-end-paragraph";
    public static final String selectionBeginParagraphAction = "selection-begin-paragraph";
    public static final String selectionEndParagraphAction = "selection-end-paragraph";
    public static final String beginAction = "caret-begin";
    public static final String endAction = "caret-end";
    public static final String selectionBeginAction = "selection-begin";
    public static final String selectionEndAction = "selection-end";
    public static final String selectWordAction = "select-word";
    public static final String selectLineAction = "select-line";
    public static final String selectParagraphAction = "select-paragraph";
    public static final String selectAllAction = "select-all";
    static final String unselectAction = "unselect";
    static final String toggleComponentOrientationAction = "toggle-componentOrientation";
    public static final String defaultKeyTypedAction = "default-typed";
    private static final Action[] defaultActions;
    
    public DefaultEditorKit() {
    }
    
    @Override
    public String getContentType() {
        return "text/plain";
    }
    
    @Override
    public ViewFactory getViewFactory() {
        return null;
    }
    
    @Override
    public Action[] getActions() {
        return (Action[])DefaultEditorKit.defaultActions.clone();
    }
    
    @Override
    public Caret createCaret() {
        return null;
    }
    
    @Override
    public Document createDefaultDocument() {
        return new PlainDocument();
    }
    
    @Override
    public void read(InputStream in, Document document, int n) throws IOException, BadLocationException {
        this.read(new InputStreamReader(in), document, n);
    }
    
    @Override
    public void write(OutputStream out, Document document, int n, int n2) throws IOException, BadLocationException {
        OutputStreamWriter outputStreamWriter = new OutputStreamWriter(out);
        this.write(outputStreamWriter, document, n, n2);
        outputStreamWriter.flush();
    }
    
    MutableAttributeSet getInputAttributes() {
        return null;
    }
    
    @Override
    public void read(Reader reader, Document document, int n) throws IOException, BadLocationException {
        char[] value = new char[4096];
        int n2 = 0;
        boolean n3 = false;
        boolean n4 = false;
        int n5 = document.getLength() == 0;
        MutableAttributeSet inputAttributes = this.getInputAttributes();
        int read;
        while ((read = reader.read(value, 0, value.length)) != -1) {
            int offset = 0;
            for (int i = 0; i < read; ++i) {
                switch (value[i]) {
                    case '\r': {
                        if (n2 != 0) {
                            n4 = true;
                            if (i == 0) {
                                document.insertString(n, "\n", inputAttributes);
                                ++n;
                                break;
                            }
                            value[i - 1] = '\n';
                            break;
                        }
                        else {
                            n2 = 1;
                            break;
                        }
                    }
                    case '\n': {
                        if (n2 != 0) {
                            if (i > offset + 1) {
                                document.insertString(n, new String(value, offset, i - offset - 1), inputAttributes);
                                n += i - offset - 1;
                            }
                            n2 = 0;
                            offset = i;
                            n3 = true;
                            break;
                        }
                        else {
                            break;
                        }
                    }
                    default: {
                        if (n2 != 0) {
                            n4 = true;
                            if (i == 0) {
                                document.insertString(n, "\n", inputAttributes);
                                ++n;
                            }
                            else {
                                value[i - 1] = '\n';
                            }
                            n2 = 0;
                            break;
                        }
                        else {
                            break;
                        }
                    }
                }
            }
            if (offset >= read) {
                continue;
            }
            if (n2 != 0) {
                if (offset >= read - 1) {
                    continue;
                }
                document.insertString(n, new String(value, offset, read - offset - 1), inputAttributes);
                n += read - offset - 1;
            }
            else {
                document.insertString(n, new String(value, offset, read - offset), inputAttributes);
                n += read - offset;
            }
        }
        if (n2 != 0) {
            document.insertString(n, "\n", inputAttributes);
            n4 = true;
        }
        if (!(n5 == 0)) {
            if (!(n3 == 0)) {
                document.putProperty("__EndOfLine__", "\r\n");
            }
            else if (!(n4 == 0)) {
                document.putProperty("__EndOfLine__", "\r");
            }
            else {
                document.putProperty("__EndOfLine__", "\n");
            }
        }
    }
    
    @Override
    public void write(Writer writer, Document document, int n, int n2) throws IOException, BadLocationException {
        if (n < 0 || n + n2 > document.getLength()) {
            throw new BadLocationException("DefaultEditorKit.write", n);
        }
        Segment segment = new Segment();
        int i = n2;
        int n3 = n;
        Object o = document.getProperty("__EndOfLine__");
        if (o == null) {
            try {
                o = System.getProperty("line.separator");
            }
            catch (SecurityException ex) {}
        }
        String str;
        if ((o instanceof String)) {
            str = (String)o;
        }
        else {
            str = null;
        }
        if (o != null && !str.equals("\n")) {
            while (i > 0) {
                int min2 = Math.min(i, 4096);
                document.getText(n3, min2, segment);
                int j = segment.offset;
                char[] array = segment.array;
                int n4;
                for (n4 = j + segment.count, ; j < n4; ++j) {
                    if (array[j] == '\n') {
                        if (j > j) {
                            writer.write(array, j, j - j);
                        }
                        writer.write(str);
                        ++j;
                    }
                }
                if (n4 > j) {
                    writer.write(array, j, n4 - j);
                }
                n3 += min2;
                i -= min2;
            }
        }
        else {
            while (i > 0) {
                int min = Math.min(i, 4096);
                document.getText(n3, min, segment);
                writer.write(segment.array, segment.offset, segment.count);
                n3 += min;
                i -= min;
            }
        }
        writer.flush();
        return;
    }
    
    static {
        defaultActions = new Action[] { new DefaultEditorKit.InsertContentAction(), new DefaultEditorKit.DeletePrevCharAction(), new DefaultEditorKit.DeleteNextCharAction(), new DefaultEditorKit.ReadOnlyAction(), new DefaultEditorKit.DeleteWordAction("delete-previous-word"), new DefaultEditorKit.DeleteWordAction("delete-next-word"), new DefaultEditorKit.WritableAction(), new DefaultEditorKit.CutAction(), new DefaultEditorKit.CopyAction(), new DefaultEditorKit.PasteAction(), new DefaultEditorKit.VerticalPageAction("page-up", -1, false), new DefaultEditorKit.VerticalPageAction("page-down", 1, false), new DefaultEditorKit.VerticalPageAction("selection-page-up", -1, true), new DefaultEditorKit.VerticalPageAction("selection-page-down", 1, true), new DefaultEditorKit.PageAction("selection-page-left", true, true), new DefaultEditorKit.PageAction("selection-page-right", false, true), new DefaultEditorKit.InsertBreakAction(), new DefaultEditorKit.BeepAction(), new DefaultEditorKit.NextVisualPositionAction("caret-forward", false, 3), new DefaultEditorKit.NextVisualPositionAction("caret-backward", false, 7), new DefaultEditorKit.NextVisualPositionAction("selection-forward", true, 3), new DefaultEditorKit.NextVisualPositionAction("selection-backward", true, 7), new DefaultEditorKit.NextVisualPositionAction("caret-up", false, 1), new DefaultEditorKit.NextVisualPositionAction("caret-down", false, 5), new DefaultEditorKit.NextVisualPositionAction("selection-up", true, 1), new DefaultEditorKit.NextVisualPositionAction("selection-down", true, 5), new DefaultEditorKit.BeginWordAction("caret-begin-word", false), new DefaultEditorKit.EndWordAction("caret-end-word", false), new DefaultEditorKit.BeginWordAction("selection-begin-word", true), new DefaultEditorKit.EndWordAction("selection-end-word", true), new DefaultEditorKit.PreviousWordAction("caret-previous-word", false), new DefaultEditorKit.NextWordAction("caret-next-word", false), new DefaultEditorKit.PreviousWordAction("selection-previous-word", true), new DefaultEditorKit.NextWordAction("selection-next-word", true), new DefaultEditorKit.BeginLineAction("caret-begin-line", false), new DefaultEditorKit.EndLineAction("caret-end-line", false), new DefaultEditorKit.BeginLineAction("selection-begin-line", true), new DefaultEditorKit.EndLineAction("selection-end-line", true), new DefaultEditorKit.BeginParagraphAction("caret-begin-paragraph", false), new DefaultEditorKit.EndParagraphAction("caret-end-paragraph", false), new DefaultEditorKit.BeginParagraphAction("selection-begin-paragraph", true), new DefaultEditorKit.EndParagraphAction("selection-end-paragraph", true), new DefaultEditorKit.BeginAction("caret-begin", false), new DefaultEditorKit.EndAction("caret-end", false), new DefaultEditorKit.BeginAction("selection-begin", true), new DefaultEditorKit.EndAction("selection-end", true), new DefaultEditorKit.DefaultKeyTypedAction(), new DefaultEditorKit.InsertTabAction(), new DefaultEditorKit.SelectWordAction(), new DefaultEditorKit.SelectLineAction(), new DefaultEditorKit.SelectParagraphAction(), new DefaultEditorKit.SelectAllAction(), new DefaultEditorKit.UnselectAction(), new DefaultEditorKit.ToggleComponentOrientationAction(), new DefaultEditorKit.DumpModelAction() };
    }
    
    public static class DefaultKeyTypedAction extends TextAction
    {
        public DefaultKeyTypedAction() {
            super("default-typed");
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JTextComponent textComponent = this.getTextComponent(actionEvent);
            if (textComponent != null && actionEvent != null) {
                if (!textComponent.isEditable() || !textComponent.isEnabled()) {
                    return;
                }
                String actionCommand = actionEvent.getActionCommand();
                int modifiers = actionEvent.getModifiers();
                if (actionCommand != null && actionCommand.length() > 0) {
                    boolean printableCharacterModifiersMask = true;
                    Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
                    if ((defaultToolkit instanceof SunToolkit)) {
                        printableCharacterModifiersMask = ((SunToolkit)defaultToolkit).isPrintableCharacterModifiersMask(modifiers);
                    }
                    int char1 = actionCommand.charAt(0);
                    if ((printableCharacterModifiersMask && char1 >= ' ' && char1 != '\u007f') || (printableCharacterModifiersMask == 0 && char1 >= '\u200c' && char1 <= '\u200d')) {
                        textComponent.replaceSelection(actionCommand);
                    }
                }
            }
        }
    }
    
    public static class InsertContentAction extends TextAction
    {
        public InsertContentAction() {
            super("insert-content");
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JTextComponent textComponent = this.getTextComponent(actionEvent);
            if (textComponent != null && actionEvent != null) {
                if (!textComponent.isEditable() || !textComponent.isEnabled()) {
                    UIManager.getLookAndFeel().provideErrorFeedback(textComponent);
                    return;
                }
                String actionCommand = actionEvent.getActionCommand();
                if (actionCommand != null) {
                    textComponent.replaceSelection(actionCommand);
                }
                else {
                    UIManager.getLookAndFeel().provideErrorFeedback(textComponent);
                }
            }
        }
    }
    
    public static class InsertBreakAction extends TextAction
    {
        public InsertBreakAction() {
            super("insert-break");
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JTextComponent textComponent = this.getTextComponent(actionEvent);
            if (textComponent != null) {
                if (!textComponent.isEditable() || !textComponent.isEnabled()) {
                    UIManager.getLookAndFeel().provideErrorFeedback(textComponent);
                    return;
                }
                textComponent.replaceSelection("\n");
            }
        }
    }
    
    public static class InsertTabAction extends TextAction
    {
        public InsertTabAction() {
            super("insert-tab");
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JTextComponent textComponent = this.getTextComponent(actionEvent);
            if (textComponent != null) {
                if (!textComponent.isEditable() || !textComponent.isEnabled()) {
                    UIManager.getLookAndFeel().provideErrorFeedback(textComponent);
                    return;
                }
                textComponent.replaceSelection("\t");
            }
        }
    }
    
    static class DeletePrevCharAction extends TextAction
    {
        DeletePrevCharAction() {
            super("delete-previous");
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JTextComponent textComponent = this.getTextComponent(actionEvent);
            boolean n = true;
            if (textComponent != null && textComponent.isEditable()) {
                try {
                    Document document = textComponent.getDocument();
                    Caret caret = textComponent.getCaret();
                    int dot = caret.getDot();
                    int mark = caret.getMark();
                    if (dot != mark) {
                        document.remove(Math.min(dot, mark), Math.abs(dot - mark));
                        n = false;
                    }
                    else if (dot > 0) {
                        int n2 = 1;
                        if (dot > 1) {
                            String text = document.getText(dot - 2, 2);
                            int char1 = text.charAt(0);
                            int char2 = text.charAt(1);
                            if (char1 >= '?' && char1 <= '?' && char2 >= '?' && char2 <= '?') {
                                n2 = 2;
                            }
                        }
                        document.remove(dot - n2, n2);
                        n = false;
                    }
                }
                catch (BadLocationException ex) {}
            }
            if (!(n == 0)) {
                UIManager.getLookAndFeel().provideErrorFeedback(textComponent);
            }
        }
    }
    
    static class DeleteNextCharAction extends TextAction
    {
        DeleteNextCharAction() {
            super("delete-next");
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JTextComponent textComponent = this.getTextComponent(actionEvent);
            boolean n = true;
            if (textComponent != null && textComponent.isEditable()) {
                try {
                    Document document = textComponent.getDocument();
                    Caret caret = textComponent.getCaret();
                    int dot = caret.getDot();
                    int mark = caret.getMark();
                    if (dot != mark) {
                        document.remove(Math.min(dot, mark), Math.abs(dot - mark));
                        n = false;
                    }
                    else if (dot < document.getLength()) {
                        int n2 = 1;
                        if (dot < document.getLength() - 1) {
                            String text = document.getText(dot, 2);
                            int char1 = text.charAt(0);
                            int char2 = text.charAt(1);
                            if (char1 >= '?' && char1 <= '?' && char2 >= '?' && char2 <= '?') {
                                n2 = 2;
                            }
                        }
                        document.remove(dot, n2);
                        n = false;
                    }
                }
                catch (BadLocationException ex) {}
            }
            if (!(n == 0)) {
                UIManager.getLookAndFeel().provideErrorFeedback(textComponent);
            }
        }
    }
    
    static class DeleteWordAction extends TextAction
    {
        DeleteWordAction(String s) {
            super(s);
            assert s == "delete-next-word";
            return;
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JTextComponent textComponent = this.getTextComponent(actionEvent);
            if (textComponent != null && actionEvent != null) {
                if (!textComponent.isEditable() || !textComponent.isEnabled()) {
                    UIManager.getLookAndFeel().provideErrorFeedback(textComponent);
                    return;
                }
                boolean n = true;
                try {
                    int selectionStart = textComponent.getSelectionStart();
                    Element paragraphElement = Utilities.getParagraphElement(textComponent, selectionStart);
                    int b;
                    if ("delete-next-word" == this.getValue("Name")) {
                        b = Utilities.getNextWordInParagraph(textComponent, paragraphElement, selectionStart, false);
                        if (b == -1) {
                            int endOffset = paragraphElement.getEndOffset();
                            if (selectionStart == endOffset - 1) {
                                b = endOffset;
                            }
                            else {
                                b = endOffset - 1;
                            }
                        }
                    }
                    else {
                        b = Utilities.getPrevWordInParagraph(textComponent, paragraphElement, selectionStart);
                        if (b == -1) {
                            int startOffset = paragraphElement.getStartOffset();
                            if (selectionStart == startOffset) {
                                b = startOffset - 1;
                            }
                            else {
                                b = startOffset;
                            }
                        }
                    }
                    int min = Math.min(selectionStart, b);
                    int abs = Math.abs(b - selectionStart);
                    if (min >= 0) {
                        textComponent.getDocument().remove(min, abs);
                        n = false;
                    }
                }
                catch (BadLocationException ex) {}
                if (!(n == 0)) {
                    UIManager.getLookAndFeel().provideErrorFeedback(textComponent);
                }
            }
        }
    }
    
    static class ReadOnlyAction extends TextAction
    {
        ReadOnlyAction() {
            super("set-read-only");
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JTextComponent textComponent = this.getTextComponent(actionEvent);
            if (textComponent != null) {
                textComponent.setEditable(false);
            }
        }
    }
    
    static class WritableAction extends TextAction
    {
        WritableAction() {
            super("set-writable");
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JTextComponent textComponent = this.getTextComponent(actionEvent);
            if (textComponent != null) {
                textComponent.setEditable(true);
            }
        }
    }
    
    public static class CutAction extends TextAction
    {
        public CutAction() {
            super("cut-to-clipboard");
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JTextComponent textComponent = this.getTextComponent(actionEvent);
            if (textComponent != null) {
                textComponent.cut();
            }
        }
    }
    
    public static class CopyAction extends TextAction
    {
        public CopyAction() {
            super("copy-to-clipboard");
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JTextComponent textComponent = this.getTextComponent(actionEvent);
            if (textComponent != null) {
                textComponent.copy();
            }
        }
    }
    
    public static class PasteAction extends TextAction
    {
        public PasteAction() {
            super("paste-from-clipboard");
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JTextComponent textComponent = this.getTextComponent(actionEvent);
            if (textComponent != null) {
                textComponent.paste();
            }
        }
    }
    
    public static class BeepAction extends TextAction
    {
        public BeepAction() {
            super("beep");
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            UIManager.getLookAndFeel().provideErrorFeedback(this.getTextComponent(actionEvent));
        }
    }
    
    static class VerticalPageAction extends TextAction
    {
        private boolean select;
        private int direction;
        
        public VerticalPageAction(String s, int direction, boolean select) {
            super(s);
            this.select = select;
            this.direction = direction;
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JTextComponent textComponent = this.getTextComponent(actionEvent);
            if (textComponent != null) {
                Rectangle visibleRect = textComponent.getVisibleRect();
                Rectangle aRect = new Rectangle(visibleRect);
                int caretPosition = textComponent.getCaretPosition();
                int n = this.direction * textComponent.getScrollableBlockIncrement(visibleRect, 1, this.direction);
                int y = visibleRect.y;
                Caret caret = textComponent.getCaret();
                Point magicCaretPosition = caret.getMagicCaretPosition();
                if (caretPosition != -1) {
                    try {
                        Rectangle modelToView = textComponent.modelToView(caretPosition);
                        int x = (magicCaretPosition != null) ? magicCaretPosition.x : modelToView.x;
                        int height = modelToView.height;
                        if (height > 0) {
                            n = n / height * height;
                        }
                        aRect.y = this.constrainY(textComponent, y + n, visibleRect.height);
                        int n2;
                        if (visibleRect.contains(modelToView.x, modelToView.y)) {
                            n2 = textComponent.viewToModel(new Point(x, this.constrainY(textComponent, modelToView.y + n, 0)));
                        }
                        else if (this.direction == -1) {
                            n2 = textComponent.viewToModel(new Point(x, aRect.y));
                        }
                        else {
                            n2 = textComponent.viewToModel(new Point(x, aRect.y + visibleRect.height));
                        }
                        int constrainOffset = this.constrainOffset(textComponent, n2);
                        if (constrainOffset != caretPosition) {
                            int adjustedY = this.getAdjustedY(textComponent, aRect, constrainOffset);
                            if ((this.direction == -1 && adjustedY <= y) || (this.direction == 1 && adjustedY >= y)) {
                                aRect.y = adjustedY;
                                if (this.select) {
                                    textComponent.moveCaretPosition(constrainOffset);
                                }
                                else {
                                    textComponent.setCaretPosition(constrainOffset);
                                }
                            }
                        }
                    }
                    catch (BadLocationException ex) {}
                }
                else {
                    aRect.y = this.constrainY(textComponent, y + n, visibleRect.height);
                }
                if (magicCaretPosition != null) {
                    caret.setMagicCaretPosition(magicCaretPosition);
                }
                textComponent.scrollRectToVisible(aRect);
            }
        }
        
        private int constrainY(JTextComponent textComponent, int max, int n) {
            if (max < 0) {
                max = 0;
            }
            else if (max + n > textComponent.getHeight()) {
                max = Math.max(0, textComponent.getHeight() - n);
            }
            return max;
        }
        
        private int constrainOffset(JTextComponent textComponent, int length) {
            Document document = textComponent.getDocument();
            if (length != 0 && length > document.getLength()) {
                length = document.getLength();
            }
            if (length < 0) {
                length = 0;
            }
            return length;
        }
        
        private int getAdjustedY(JTextComponent textComponent, Rectangle rectangle, int n) {
            int n2 = rectangle.y;
            try {
                Rectangle modelToView = textComponent.modelToView(n);
                if (modelToView.y < rectangle.y) {
                    n2 = modelToView.y;
                }
                else if (modelToView.y > rectangle.y + rectangle.height || modelToView.y + modelToView.height > rectangle.y + rectangle.height) {
                    n2 = modelToView.y + modelToView.height - rectangle.height;
                }
            }
            catch (BadLocationException ex) {}
            return n2;
        }
    }
    
    static class PageAction extends TextAction
    {
        private boolean select;
        private boolean left;
        
        public PageAction(String s, boolean left, boolean select) {
            super(s);
            this.select = select;
            this.left = left;
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JTextComponent textComponent = this.getTextComponent(actionEvent);
            if (textComponent != null) {
                Rectangle visibleRect = new Rectangle();
                textComponent.computeVisibleRect(visibleRect);
                if (this.left) {
                    visibleRect.x = Math.max(0, visibleRect.x - visibleRect.width);
                }
                else {
                    visibleRect.x += visibleRect.width;
                }
                if (textComponent.getCaretPosition() != -1) {
                    int caretPosition;
                    if (this.left) {
                        caretPosition = textComponent.viewToModel(new Point(visibleRect.x, visibleRect.y));
                    }
                    else {
                        caretPosition = textComponent.viewToModel(new Point(visibleRect.x + visibleRect.width - 1, visibleRect.y + visibleRect.height - 1));
                    }
                    Document document = textComponent.getDocument();
                    if (caretPosition != 0 && caretPosition > document.getLength() - 1) {
                        caretPosition = document.getLength() - 1;
                    }
                    else if (caretPosition < 0) {
                        caretPosition = 0;
                    }
                    if (this.select) {
                        textComponent.moveCaretPosition(caretPosition);
                    }
                    else {
                        textComponent.setCaretPosition(caretPosition);
                    }
                }
            }
        }
    }
    
    static class DumpModelAction extends TextAction
    {
        DumpModelAction() {
            super("dump-model");
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JTextComponent textComponent = this.getTextComponent(actionEvent);
            if (textComponent != null) {
                Document document = textComponent.getDocument();
                if ((document instanceof AbstractDocument)) {
                    ((AbstractDocument)document).dump(System.err);
                }
            }
        }
    }
    
    static class NextVisualPositionAction extends TextAction
    {
        private boolean select;
        private int direction;
        
        NextVisualPositionAction(String s, boolean select, int direction) {
            super(s);
            this.select = select;
            this.direction = direction;
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JTextComponent textComponent = this.getTextComponent(actionEvent);
            if (textComponent != null) {
                Caret caret = textComponent.getCaret();
                DefaultCaret defaultCaret = (caret instanceof DefaultCaret) ? ((DefaultCaret)caret) : null;
                int dot = caret.getDot();
                Position.Bias[] array = { null };
                Point magicCaretPosition = caret.getMagicCaretPosition();
                try {
                    if (magicCaretPosition == null && (this.direction == 1 || this.direction == 5)) {
                        Rectangle rectangle = (defaultCaret != null) ? textComponent.getUI().modelToView(textComponent, dot, defaultCaret.getDotBias()) : textComponent.modelToView(dot);
                        magicCaretPosition = new Point(rectangle.x, rectangle.y);
                    }
                    NavigationFilter navigationFilter = textComponent.getNavigationFilter();
                    int dot2;
                    if (navigationFilter != null) {
                        dot2 = navigationFilter.getNextVisualPositionFrom(textComponent, dot, (defaultCaret != null) ? defaultCaret.getDotBias() : Position.Bias.Forward, this.direction, array);
                    }
                    else {
                        dot2 = textComponent.getUI().getNextVisualPositionFrom(textComponent, dot, (defaultCaret != null) ? defaultCaret.getDotBias() : Position.Bias.Forward, this.direction, array);
                    }
                    if (array[0] == null) {
                        array[0] = Position.Bias.Forward;
                    }
                    if (defaultCaret != null) {
                        if (this.select) {
                            defaultCaret.moveDot(dot2, array[0]);
                        }
                        else {
                            defaultCaret.setDot(dot2, array[0]);
                        }
                    }
                    else if (this.select) {
                        caret.moveDot(dot2);
                    }
                    else {
                        caret.setDot(dot2);
                    }
                    if (magicCaretPosition != null && (this.direction == 1 || this.direction == 5)) {
                        textComponent.getCaret().setMagicCaretPosition(magicCaretPosition);
                    }
                }
                catch (BadLocationException ex) {}
            }
        }
    }
    
    static class BeginWordAction extends TextAction
    {
        private boolean select;
        
        BeginWordAction(String s, boolean select) {
            super(s);
            this.select = select;
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JTextComponent textComponent = this.getTextComponent(actionEvent);
            if (textComponent != null) {
                try {
                    int wordStart = Utilities.getWordStart(textComponent, textComponent.getCaretPosition());
                    if (this.select) {
                        textComponent.moveCaretPosition(wordStart);
                    }
                    else {
                        textComponent.setCaretPosition(wordStart);
                    }
                }
                catch (BadLocationException ex) {
                    UIManager.getLookAndFeel().provideErrorFeedback(textComponent);
                }
            }
        }
    }
    
    static class EndWordAction extends TextAction
    {
        private boolean select;
        
        EndWordAction(String s, boolean select) {
            super(s);
            this.select = select;
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JTextComponent textComponent = this.getTextComponent(actionEvent);
            if (textComponent != null) {
                try {
                    int wordEnd = Utilities.getWordEnd(textComponent, textComponent.getCaretPosition());
                    if (this.select) {
                        textComponent.moveCaretPosition(wordEnd);
                    }
                    else {
                        textComponent.setCaretPosition(wordEnd);
                    }
                }
                catch (BadLocationException ex) {
                    UIManager.getLookAndFeel().provideErrorFeedback(textComponent);
                }
            }
        }
    }
    
    static class PreviousWordAction extends TextAction
    {
        private boolean select;
        
        PreviousWordAction(String s, boolean select) {
            super(s);
            this.select = select;
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JTextComponent textComponent = this.getTextComponent(actionEvent);
            if (textComponent != null) {
                int caretPosition = textComponent.getCaretPosition();
                boolean n = false;
                try {
                    Element paragraphElement = Utilities.getParagraphElement(textComponent, caretPosition);
                    caretPosition = Utilities.getPreviousWord(textComponent, caretPosition);
                    if (caretPosition < paragraphElement.getStartOffset()) {
                        caretPosition = Utilities.getParagraphElement(textComponent, caretPosition).getEndOffset() - 1;
                    }
                }
                catch (BadLocationException ex) {
                    if (caretPosition != 0) {
                        caretPosition = 0;
                    }
                    else {
                        n = true;
                    }
                }
                if (n == 0) {
                    if (this.select) {
                        textComponent.moveCaretPosition(caretPosition);
                    }
                    else {
                        textComponent.setCaretPosition(caretPosition);
                    }
                }
                else {
                    UIManager.getLookAndFeel().provideErrorFeedback(textComponent);
                }
            }
        }
    }
    
    static class NextWordAction extends TextAction
    {
        private boolean select;
        
        NextWordAction(String s, boolean select) {
            super(s);
            this.select = select;
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JTextComponent textComponent = this.getTextComponent(actionEvent);
            if (textComponent != null) {
                int caretPosition = textComponent.getCaretPosition();
                boolean n = false;
                Element paragraphElement = Utilities.getParagraphElement(textComponent, caretPosition);
                try {
                    caretPosition = Utilities.getNextWord(textComponent, caretPosition);
                    if (caretPosition >= paragraphElement.getEndOffset() && caretPosition != paragraphElement.getEndOffset() - 1) {
                        caretPosition = paragraphElement.getEndOffset() - 1;
                    }
                }
                catch (BadLocationException ex) {
                    int length = textComponent.getDocument().getLength();
                    if (caretPosition != length) {
                        if (caretPosition != paragraphElement.getEndOffset() - 1) {
                            caretPosition = paragraphElement.getEndOffset() - 1;
                        }
                        else {
                            caretPosition = length;
                        }
                    }
                    else {
                        n = true;
                    }
                }
                if (n == 0) {
                    if (this.select) {
                        textComponent.moveCaretPosition(caretPosition);
                    }
                    else {
                        textComponent.setCaretPosition(caretPosition);
                    }
                }
                else {
                    UIManager.getLookAndFeel().provideErrorFeedback(textComponent);
                }
            }
        }
    }
    
    static class BeginLineAction extends TextAction
    {
        private boolean select;
        
        BeginLineAction(String s, boolean select) {
            super(s);
            this.select = select;
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JTextComponent textComponent = this.getTextComponent(actionEvent);
            if (textComponent != null) {
                try {
                    int rowStart = Utilities.getRowStart(textComponent, textComponent.getCaretPosition());
                    if (this.select) {
                        textComponent.moveCaretPosition(rowStart);
                    }
                    else {
                        textComponent.setCaretPosition(rowStart);
                    }
                }
                catch (BadLocationException ex) {
                    UIManager.getLookAndFeel().provideErrorFeedback(textComponent);
                }
            }
        }
    }
    
    static class EndLineAction extends TextAction
    {
        private boolean select;
        
        EndLineAction(String s, boolean select) {
            super(s);
            this.select = select;
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JTextComponent textComponent = this.getTextComponent(actionEvent);
            if (textComponent != null) {
                try {
                    int rowEnd = Utilities.getRowEnd(textComponent, textComponent.getCaretPosition());
                    if (this.select) {
                        textComponent.moveCaretPosition(rowEnd);
                    }
                    else {
                        textComponent.setCaretPosition(rowEnd);
                    }
                }
                catch (BadLocationException ex) {
                    UIManager.getLookAndFeel().provideErrorFeedback(textComponent);
                }
            }
        }
    }
    
    static class BeginParagraphAction extends TextAction
    {
        private boolean select;
        
        BeginParagraphAction(String s, boolean select) {
            super(s);
            this.select = select;
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JTextComponent textComponent = this.getTextComponent(actionEvent);
            if (textComponent != null) {
                int startOffset = Utilities.getParagraphElement(textComponent, textComponent.getCaretPosition()).getStartOffset();
                if (this.select) {
                    textComponent.moveCaretPosition(startOffset);
                }
                else {
                    textComponent.setCaretPosition(startOffset);
                }
            }
        }
    }
    
    static class EndParagraphAction extends TextAction
    {
        private boolean select;
        
        EndParagraphAction(String s, boolean select) {
            super(s);
            this.select = select;
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JTextComponent textComponent = this.getTextComponent(actionEvent);
            if (textComponent != null) {
                int min = Math.min(textComponent.getDocument().getLength(), Utilities.getParagraphElement(textComponent, textComponent.getCaretPosition()).getEndOffset());
                if (this.select) {
                    textComponent.moveCaretPosition(min);
                }
                else {
                    textComponent.setCaretPosition(min);
                }
            }
        }
    }
    
    static class BeginAction extends TextAction
    {
        private boolean select;
        
        BeginAction(String s, boolean select) {
            super(s);
            this.select = select;
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JTextComponent textComponent = this.getTextComponent(actionEvent);
            if (textComponent != null) {
                if (this.select) {
                    textComponent.moveCaretPosition(0);
                }
                else {
                    textComponent.setCaretPosition(0);
                }
            }
        }
    }
    
    static class EndAction extends TextAction
    {
        private boolean select;
        
        EndAction(String s, boolean select) {
            super(s);
            this.select = select;
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JTextComponent textComponent = this.getTextComponent(actionEvent);
            if (textComponent != null) {
                int length = textComponent.getDocument().getLength();
                if (this.select) {
                    textComponent.moveCaretPosition(length);
                }
                else {
                    textComponent.setCaretPosition(length);
                }
            }
        }
    }
    
    static class SelectWordAction extends TextAction
    {
        private Action start;
        private Action end;
        
        SelectWordAction() {
            super("select-word");
            this.start = new DefaultEditorKit.BeginWordAction("pigdog", false);
            this.end = new DefaultEditorKit.EndWordAction("pigdog", true);
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            this.start.actionPerformed(actionEvent);
            this.end.actionPerformed(actionEvent);
        }
    }
    
    static class SelectLineAction extends TextAction
    {
        private Action start;
        private Action end;
        
        SelectLineAction() {
            super("select-line");
            this.start = new DefaultEditorKit.BeginLineAction("pigdog", false);
            this.end = new DefaultEditorKit.EndLineAction("pigdog", true);
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            this.start.actionPerformed(actionEvent);
            this.end.actionPerformed(actionEvent);
        }
    }
    
    static class SelectParagraphAction extends TextAction
    {
        private Action start;
        private Action end;
        
        SelectParagraphAction() {
            super("select-paragraph");
            this.start = new DefaultEditorKit.BeginParagraphAction("pigdog", false);
            this.end = new DefaultEditorKit.EndParagraphAction("pigdog", true);
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            this.start.actionPerformed(actionEvent);
            this.end.actionPerformed(actionEvent);
        }
    }
    
    static class SelectAllAction extends TextAction
    {
        SelectAllAction() {
            super("select-all");
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JTextComponent textComponent = this.getTextComponent(actionEvent);
            if (textComponent != null) {
                Document document = textComponent.getDocument();
                textComponent.setCaretPosition(0);
                textComponent.moveCaretPosition(document.getLength());
            }
        }
    }
    
    static class UnselectAction extends TextAction
    {
        UnselectAction() {
            super("unselect");
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JTextComponent textComponent = this.getTextComponent(actionEvent);
            if (textComponent != null) {
                textComponent.setCaretPosition(textComponent.getCaretPosition());
            }
        }
    }
    
    static class ToggleComponentOrientationAction extends TextAction
    {
        ToggleComponentOrientationAction() {
            super("toggle-componentOrientation");
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JTextComponent textComponent = this.getTextComponent(actionEvent);
            if (textComponent != null) {
                ComponentOrientation componentOrientation;
                if (textComponent.getComponentOrientation() == ComponentOrientation.RIGHT_TO_LEFT) {
                    componentOrientation = ComponentOrientation.LEFT_TO_RIGHT;
                }
                else {
                    componentOrientation = ComponentOrientation.RIGHT_TO_LEFT;
                }
                textComponent.setComponentOrientation(componentOrientation);
                textComponent.repaint();
            }
        }
    }
}
